
# Third party ICMP class and module.

import struct, warnings

warnings.filterwarnings("ignore",category=DeprecationWarning)

class Packet(object):

    """Creates ICMPv4 and v6 packets.
    
    header
        two-item sequence containing the type and code of the packet,
        respectively.
    version
        Automatically set to version of protocol being used or None if ambiguous.
    data
        Contains data of the packet.  Can only assign a subclass of basestring
        or None.

    packet
        binary representation of packet.
    
    """

    header_table = {
                0 : (0, 4),
                #3 : (15, 4),  Overlap with ICMPv6
                3 : (15, None),
                #4 : (0, 4),  Deprecated by RFC 1812
                5 : (3, 4),
                8 : (0, 4),
                9 : (0, 4),
                10: (0, 4),
                11: (1, 4),
                12: (1, 4),
                13: (0, 4),
                14: (0, 4),
                15: (0, 4),
                16: (0, 4),
                17: (0, 4),
                18: (0, 4),

                1 : (4, 6),
                2 : (0, 6),
                #3 : (2, 6),  Overlap with ICMPv4
                #4 : (2, 6),  Type of 4 in ICMPv4 is deprecated
                4 : (2, None),
                128: (0, 6),
                129: (0, 6),
                130: (0, 6),
                131: (0, 6),
                132: (0, 6),
                133: (0, 6),
                134: (0, 6),
                135: (0, 6),
                136: (0, 6),
                137: (0, 6),
             }

    def _setheader(self, header):
        """Set type, code, and version for the packet."""
        if len(header) != 2:
            raise ValueError("header data must be in a two-item sequence")
        type_, code = header
        try:
            max_range, version = self.header_table[type_]
        except KeyError:
            raise ValueError("%s is not a valid type argument" % type_)
        else:
            if code > max_range:
                raise ValueError("%s is not a valid code value for type %s" %\
                                     (type_, code))
            self._type, self._code, self._version = type_, code, version

    header = property(lambda self: (self._type, self._code), _setheader,
                       doc="type and code of packet")

    version = property(lambda self: self._version,
                        doc="Protocol version packet is using or None if "
                            "ambiguous")

    def _setdata(self, data):
        """Setter for self.data; will only accept a basestring or None type."""
        if not isinstance(data, basestring) and not isinstance(data, type(None)):
            raise TypeError("value must be a subclass of basestring or None, "
                            "not %s" % type(data))
        self._data = data

    data = property(lambda self: self._data, _setdata,
                    doc="data contained within the packet")

    def __init__(self, header=(None, None), data=None):
        """Set instance attributes if given."""
        #XXX: Consider using __slots__
        # self._version initialized by setting self.header
        self.header = header
        self.data = data

    def __repr__(self):
        return "<ICMPv%s packet: type = %s, code = %s, data length = %s>" % \
                (self.version, self.type, self.code, len(self.data))

    def create(self):
        """Return a packet."""
        # Kept as a separate method instead of rolling into 'packet' property so
        # as to allow passing method around without having to define a lambda
        # method.
        args = [self.header[0], self.header[1], 0]
        pack_format = "!BBH"
        if self.data:
            pack_format += "%ss" % len(self.data)
            args.append(self.data)
        # ICMPv6 has the IP stack calculate the checksum
        # For ambiguous cases, just go ahead and calculate it just in case
        if self.version == 4 or not self.version:
            args[2] = self._checksum(struct.pack(pack_format, *args))
        return struct.pack(pack_format, *args)

    packet = property(create,
                       doc="Complete ICMP packet")

    def _checksum(self, checksum_packet):
        """Calculate checksum"""
        byte_count = len(checksum_packet)
        #XXX: Think there is an error here about odd number of bytes
        if byte_count % 2:
            odd_byte = ord(checksum_packet[-1])
            checksum_packet = checksum_packet[:-1]
        else:
            odd_byte = 0
        two_byte_chunks = struct.unpack("!%sH" % (len(checksum_packet)/2),
                                        checksum_packet)
        total = 0
        for two_bytes in two_byte_chunks:
            total += two_bytes
        else:
            total += odd_byte
        total = (total >> 16) + (total & 0xFFFF)
        total += total >> 16
        return ~total
        
    def parse(cls, packet):
        """Parse ICMP packet and return an instance of Packet"""
        string_len = len(packet) - 4 # Ignore IP header
        pack_format = "!BBH"
        if string_len:
            pack_format += "%ss" % string_len
        unpacked_packet = struct.unpack(pack_format, packet)
        type, code, checksum = unpacked_packet[:3]
        try:
            data = unpacked_packet[3]
        except IndexError:
            data = None
        return cls((type, code), data)

    parse = classmethod(parse)


#------------
# ping.py
#------------

import struct,socket,sys,time, os
#from icmplib import Packet

def main(addr):
    s = socket.socket(socket.AF_INET,socket.SOCK_RAW)
    s.connect((addr,))

datalen = 56
BUFSIZE = 1500

# Must be sudo (root user) to ping hosts.
def ping(addr, timeout):

    ## create socket
    s = socket.socket(socket.AF_INET,socket.SOCK_RAW,
                        socket.getprotobyname('icmp'))
    
    s.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
    s.setsockopt(socket.SOL_SOCKET, socket.SO_BROADCAST, 1)
    s.settimeout(timeout)

    s.connect((addr,22))

    ## setuid back to normal user
    os.setuid(os.getuid())

    seq_num = 0
    packet_count = 0
    process_id = os.getpid()
    base_packet = Packet((8,0))

    ## create ping packet 
    seq_num += 1
    pdata = struct.pack("!HHd",process_id,seq_num,time.time())
    
    ## send initial packet 
    base_packet.data = pdata
    s.send(base_packet.packet)
    
    ## recv packet, returns recv status
    try:
        buf = s.recv(BUFSIZE)
        return True
    except socket.error:
        return False
        
if __name__=='__main__':
    import sys
    ping(sys.argv[1])

# End of script.




# Network module used for all UDP networking aspects of the Gatecrasher script.
# Developed by Michael Telford.

import socket

# Initializes socket with datagram proto and binds to port arg.
def bind(port):

    global s
    host = ''

    s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    s.bind((host, port))

def send(addr, port):

    global s
    s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    data = "gatecrasher request"
    address = (addr, port)

    s.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
    s.setsockopt(socket.SOL_SOCKET, socket.SO_BROADCAST, 1)

    s.sendto(data, address)

def receive(timeout):
    
    global s

    s.settimeout(timeout)
    while 1:
        try:
            string, address = s.recvfrom(1024)
            return True
        except socket.timeout:
            return False

def receive_echo(port):
    
    global s
    s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)

    s.bind(('', port))
    # Block until receive and then echo loop (continuous).
    while 1:
        string, address = s.recvfrom(1024)
        s.sendto(string, address)

def close():

    global s
    try:
        s.shutdown(socket.SHUT_RDWR)
        s.close()
    except socket.error:
        pass

# End of module.





# Network module used for all TCP networking aspects of the Gatecrasher script.
# Developed by Michael Telford.

import socket

# Initializes socket with stream proto and binds to port arg.
def bind(port):

    global s
    host = ''

    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    s.bind((host, port))

def connect(addr, port, timeout):

    global s
    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    address = (addr, port)

    s.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
    s.setsockopt(socket.SOL_SOCKET, socket.SO_BROADCAST, 1)
    s.settimeout(timeout)
    try:
        s.connect(address)
        close()
        return True
    except (socket.error, socket.timeout):
        return False

def listen(timeout, num_connections):

    global s
    s.settimeout(timeout)
    while 1:
        try:
            s.listen(num_connections)
            conn, addr = s.accept()
            a.append(addr)
        except socket.timeout:
            a.sort()
            return a

def close():

    global s
    try:
        s.shutdown(socket.SHUT_RDWR)
        s.close()
    except socket.error:
        pass

# End of module.





# Gatecrasher script to find 'awake' hosts running different protocol services.
# Developed by Michael Telford.

import socket, sys, time           # Standard modules
import UDP, TCP, ICMP              # Created modules

# Check user access for ping functionality.
def check_ping_priv():
    try:
        ICMP.ping("localhost", 0.01)
    except socket.error:
        print("Must have root (administrator) access for ICMP, Goodbye")
        sys.exit(0)

# Ping hosts on network using the network ID arg.
def ping(netid, tout):
    i = 1
    a = []

    check_ping_priv()

    print("Pinging hosts now, remember that patience is a virtue...")
    try:
        stime = time.time()
        while i < 255:
            host = netid + "." + str(i)
            if ICMP.ping(host, tout):
                a.append(host)
            i += 1
        etime = time.time()
        duration = etime - stime
    except socket.error:
        print("A socket error has occured, exiting")
        sys.exit(0)
    except KeyboardInterrupt:
        sys.exit(0)

    # Print awake hosts, if any.
    print("Tests took " + str(int(duration)) + " seconds with a " + str(tout) + " timeout")
    if len(a) > 0:
        print("The following hosts are 'awake' : ")
        # Do not sort array, it's already done due to i var being incremented.
        for x in a:
            print(x)
    else:
        print("No hosts are awake or they're just not responding to ICMP ping requests")

# Check for servers running TCP based software such as HTTP and HTTPS etc...
def tcp_conn(netid, port, tout):
    global proto
    i = 1
    a = []

    print("Gathering host IP's now, remember that patience is a virtue...")
    try:
        stime = time.time()
        while i < 255:
            host = netid + "." + str(i)
            if TCP.connect(host, port, tout):
                a.append(host)
            i += 1
        etime = time.time()
        duration = etime - stime
    except socket.error:
        print("A socket error has occured, exiting")
        sys.exit(0)
    except KeyboardInterrupt:
        sys.exit(0)

    # Print awake hosts, if any.
    print("Tests took " + str(int(duration)) + " seconds with a " + str(tout) + " timeout")
    if len(a) > 0:
        print("The following hosts are running " + proto +  " server software :")
        # Do not sort array, it's already done due to i var being incremented.
        for x in a:
            print(x)
    else:
        print("No hosts are awake or they're just not responding to " + proto + " requests")

def udp_send_receive(netid, port, tout):
    i = 1
    a = []

    print("Gathering host IP's now, remember that patience is a virtue...")
    try:
        stime = time.time()
        while i < 255:
            host = netid + "." + str(i)
            UDP.send(host, port)
            if UDP.receive(tout):
                a.append(host)
            i += 1
        etime = time.time()
        duration = etime - stime
    except socket.error:
        print("A socket error has occured, exiting")
        sys.exit(0)
    except KeyboardInterrupt:
        sys.exit(0)

    # Print awake hosts, if any.
    print("Tests took " + str(int(duration)) + " seconds with a " + str(tout) + " timeout")
    if len(a) > 0:
        print("The following hosts responded to the UDP packet sent :")
        # Do not sort array, it's already done due to i var being incremented.
        for x in a:
            print(x)
    else:
        print("No hosts are awake or they're just not responding to UDP packets")

def raw_data(netid, proto, tout):
    # Obtain the correct protocol and port
    p = proto
    port = p[p.find(":")+1 : p.find("#")]
    proto = p[p.find("@")+1 : p.find(":")]
    
    # Perform raw data transmissions
    if proto == "tcp" or proto == "TCP":
        tcp_conn(netid, int(port), tout)
    elif proto == "udp" or proto == "UDP":
        udp_send_receive(netid, int(port), tout)
    else:
        print("Incorrect argument usage, see help below")
        print_help()
        sys.exit(0)

# Prints usage help to user if incorrect num of args is provided.
def print_help():
    print("""
Gatecrasher can be thought of as a subnet broadcast tool which records 
contactable host addresses and displays them to the user.  Different protocols
and ports can be used to contact potential hosts.

3 user arguments can be provided but only 1 is needed.  The full 3 args consist 
of a network ID string, a protocol or service string, and lastly a timeout 
integer or floating point integer for connections and replies depending on
the nature of the selected service.
Only the network ID is needed meaning the protocol and timeout are taken as the 
default values set by the script which is the 'ICMP' (ping) protocol and 0.25 
second timeout to listen for replies if not provided as arguments.
Greater timeouts may be needed on WAN's for accurate results.

Custom transport layer protocols and port combinations can be used to test
contactable hosts on a given network ID.
Gatecrasher also has a built in UDP echo service which binds to a given port
and echos any data sent to it until interrupted by the user manually.
See usage examples below.

Usage examples :

gatecrasher.py 192.168.1                - Pings all hosts on network ID
gatecrasher.py 172.4.208 http           - Checks for all web servers
gatecrasher.py 10.2.128 ssh 0.7         - Checks for all ssh servers
gatecrasher.py 82.4.207 @tcp:8080@      - Custom TCP tomcat search
gatecrasher.py 192.168.0 @udp:67@ 0.05  - Custom UDP network search
gatecrasher.py echo 8888                - UDP echo server on port 8888

Standard supported protocols include :
ICMP, HTTP, HTTPS, FTP, FTPS, SSH, MySQL\n""")

def main():
	# Default variables if not set via user args.
	netid = None
	proto = 'ICMP'
	tout = 0.25
	
	# User arguments.
	if len(sys.argv) == 2:
		if sys.argv[1] == '-h' or sys.argv[1] == '-help' or sys.argv[1] == '--help':
			print_help()
			sys.exit(0)
		else:
			netid = sys.argv[1]
	elif len(sys.argv) == 3:
		if sys.argv[1] == "echo" or sys.argv[1] == "ECHO":
			print("Echoing UDP data now...")
			try:
				UDP.receive_echo(int(sys.argv[2]))
			except KeyboardInterrupt:
				sys.exit(0)
			except socket.error:
				print("Socket error, check user permissions")
				sys.exit(0)
		else:
			netid = sys.argv[1]
			proto = sys.argv[2]
	elif len(sys.argv) == 4:
		netid = sys.argv[1]
		proto = sys.argv[2]
		tout = float(sys.argv[3])
	else:
		print("Incorrect argument usage, see help below")
		print_help()
		sys.exit(0)
	
	# Perform network search for hosts on given protocol/service.
	if proto == 'icmp' or proto == 'ICMP' or proto == 'ping':
		ping(netid, tout)
	elif proto == 'http' or proto == 'HTTP':
		tcp_conn(netid, 80, tout)
	elif proto == 'https' or proto == 'HTTPS':
		tcp_conn(netid, 443, tout)
	elif proto == 'ftp' or proto == 'FTP':
		tcp_conn(netid, 21, tout)
	elif proto == 'ftps' or proto == 'FTPS':
		tcp_conn(netid, 990, tout)
	elif proto == 'ssh' or proto == 'SSH':
		tcp_conn(netid, 22, tout)
	elif proto == 'mysql' or proto == 'MYSQL' or proto == 'MySQL':
		tcp_conn(netid, 3306, tout)
	elif proto.find("@") != -1:
		raw_data(netid, proto, tout)
	else:
		print("Incorrect argument usage, see help below")
		print_help()
		sys.exit()
	
	#raw_input("Press any key to exit...")
	exit(1)

if __name__ == '__main__':
    main()
